#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # 2A.soft - Jupyter et commandes magiques
# 
# Pour être inventif, il faut être un peu paresseux. Cela explique parfois la syntaxe peu compréhensible mais réduite de certaines instructions. Cela explique sans doute aussi que [Jupyter](http://jupyter.org/) offre la possibilité de définir des [commandes magiques](http://ipython.readthedocs.io/en/stable/config/custommagics.html) qu'on peut interpréter comme des raccourcis. ``%`` pour une ligne, ``%%`` pour une cellule.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Commande magique
# 
# Ce sont des raccourcis. Si vous n'avez plus envie d'écrire le même code tous les jours alors peut-être que vous avez envie de créer une commande magique. C'est une fonctionnalité des notebooks Jupyter. L'exemple suivant crée une commande magique qui génère une séquence de nombre aléatoire dans le notebook. On l'appelle ``RND``. Cela se passe en trois étapes :
# 
# * implémentation de la commande
# * déclaration de la commande (pour Jupyter)
# * utilisation de la commande
# 
# C'est l'objet des trois cellules qui suivent.

# In[2]:


import random
from IPython.core.magic import Magics, magics_class, line_magic, cell_magic, line_cell_magic
 
@magics_class
class CustomMagics(Magics):
 
    @line_magic
    def RND(self, line):
        return [ random.random() for i in range(0,int(line))]


# In[3]:


ip = get_ipython()
ip.register_magics(CustomMagics)


# In[4]:


get_ipython().run_line_magic('RND', '20')


# ## Exercice 1 : votre propre magique commande
# 
# Je voulais vous demander comme exercice d'écrire une commande magique qui [envoie un mail](http://en.wikibooks.org/wiki/Python_Programming/Email). On pourrait imaginer une commande qui récupère les messages facebook. Finalement, ce sera une commande magique ``%plot`` qui dessine le graphe de vos rêves. Vous pourrez perfectionner votre commande magique en lisant cette page [Registering callbacks](http://jupyter-notebook.readthedocs.io/en/latest/examples/Notebook/JavaScript%20Notebook%20Extensions.html#Register-a-callback).
# 
# **Remarque :** Si vous décidez d'automatiser l'envoi des mails, prenez garde de ne pas écrire votre mot de passe dans le code de votre programme surtout si son code est disponible sur [GitHub](https://github.com/). Il est préférable de le demander à chaque fois ou de le stocker de façon cryptée :
# 
# * [django](https://github.com/django/django/blob/master/django/contrib/auth/hashers.py#L216)
# * [keyring](https://bitbucket.org/kang/python-keyring-lib)

# In[5]: